/*
			  Program Name : Basic_Files
			  Package Name : Subroutine Library - Layer 0
			  Purpose      : Basic File classes
			  Institution  : Conroy & Conroy Co.
			  Date Written : 14-Feb-2021
              Written By   : Alan Conroy
              Version      : 2.0

              TO THE GLORY OF GOD THROUGH JESUS CHRIST

        Copyright (C) 2021 by Conroy & Conroy.  Released to the public domain.

        *********************************************************
        *                                                       *
        *        M O D I F I C A T I O N   H I S T O R Y        *
        *                                                       *
        *********************************************************

            DATE      BY             REASON

        *********************************************************
		*                                                       *
		*           P R O G R A M   P U R P O S E               *
		*                                                       *
		*********************************************************

		This unit has simplified base file functions and classes suitable for
		use in Subroutine Layer 0.
*/

#include "Basic_Files.hpp";

using namespace std ;

bool FileExists( const string& name )
{
	ifstream f( name.c_str() ) ;
	return f.good() ;
}


PSBasic_File Open_Text_File( const string Name, int Flags, int& Err )
{
	Err = 0 ;
	if( ( Flags & FOF_Must_Exist ) != 0 )
	{
		if( !FileExists( Name ) )
		{
			Err = FErr_Not_Found ;
			return NULL ;
		} ;
	} ;

	PSBasic_File Fi = new TSBasic_File() ;
	Err = Fi->Open( Name, Flags ) ;
	if( Err != 0 )
	{
		delete Fi ;
		return NULL ;
	} ;
	return Fi ;
} ;


// TSBasic_File method...

int TSBasic_File::Open( const string Name, int Flags )
{
	int E = 0 ;

	if( ( Flags & FOF_Must_Exist ) != 0 )
	{
		if( !FileExists( Name ) )
		{
			return FErr_Not_Found ;
		} ;
	} ;
	if( ( Flags & FOF_Append ) != 0 )
	{
		F.open( Name.c_str(), std::ios::app );
	}
	else
	if( ( Flags & FOF_Create ) != 0 )
	{
		F.open( Name.c_str(), std::ios::trunc );
	}
	else
	{
		F.open( Name.c_str(), ios::in | ios::out );
	} ;
	_Name = Name ;
	return E ;
} ;


void TSBasic_File::Close()
{
	F.close() ;
} ;


bool TSBasic_File::eof()
{
	return F.eof() ;
} ;


string TSBasic_File::FileName()
{
	return _Name ;
} ;


string TSBasic_File::ReadLn()
{
	string S ;

	getline( F, S ) ;
	return S ;
} ;


void TSBasic_File::WriteLn( const string Value )
{
	F << Value << '\n';
} ;

