using System;
using System.IO;

namespace Basic_Files_unit
{

	/*
				  Program Name : Basic_Files
				  Package Name : Subroutine Library - Layer 0
				  Purpose      : Basic File classes
				  Institution  : Conroy & Conroy Co.
				  Date Written : 14-Feb-2021
				  Written By   : Alan Conroy
				  Version      : 2.0

				  TO THE GLORY OF GOD THROUGH JESUS CHRIST

			Copyright (C) 2021 by Conroy & Conroy.  Released to the public domain.

			*********************************************************
			*                                                       *
			*        M O D I F I C A T I O N   H I S T O R Y        *
			*                                                       *
			*********************************************************

				DATE      BY             REASON

			*********************************************************
			*                                                       *
			*           P R O G R A M   P U R P O S E               *
			*                                                       *
			*********************************************************

			This unit has simplified base file functions and classes suitable for
			use in Subroutine Layer 0.
	*/

	public class TSBasic_File
	{
		// File open flags
		public const int FOF_Read_Write = 0 ; // Open for reading and writing
		public const int FOF_Read_Only = 1 ; // Open for reading only
		public const int FOF_Append = 2 ; // Open and position at end of file
		public const int FOF_Must_Exist = 4 ; // File must exist
		public const int FOF_Create = 8 ; // Create new file/overwrite old file

		public const int FErr_Not_Found = 1 ; // File doesn't exist

		
		private FileStream F ;
		private string _Name ;
		private TextReader sr;
		private TextWriter sw;

		public bool FileExists( string name )
		{
			return File.Exists(name) ;
		}


		public int Open( string Name, int Flags )
		{
			if( ( Flags & FOF_Must_Exist ) != 0 )
			{
				if( !FileExists( Name ) )
				{
					return FErr_Not_Found ;
				} ;
			} ;
			int err = 0;
			try
			{
				if( ( Flags & FOF_Append ) != 0 )
				{
					if( ( Flags & FOF_Read_Only ) != 0 )
					{
						F = new FileStream( Name, FileMode.Append, FileAccess.Read,
							FileShare.Read ) ;
					}
					else
					{
						F = new FileStream( Name, FileMode.Append, FileAccess.ReadWrite,
							FileShare.Read ) ;
						sw = new StreamWriter(F);
					}
				}
				else
				if( ( Flags & FOF_Create ) != 0 )
				{
					F = new FileStream( Name, FileMode.CreateNew, FileAccess.ReadWrite,
						FileShare.Read ) ;
					sw = new StreamWriter(F);
				}
				else
				{
					if( ( Flags & FOF_Read_Only ) != 0 )
					{
						F = new FileStream( Name, FileMode.Open, FileAccess.Read,
							FileShare.Read ) ;
					}
					else
					{
						F = new FileStream( Name, FileMode.Open, FileAccess.ReadWrite,
							FileShare.Read ) ;
						sw = new StreamWriter(F);
					}
				} ;
				sr = new StreamReader(F);
				_Name = Name ;
			}
			catch(Exception e)
			{
				err=2 ;
			}
			return err ;
		}


		public void Close()
		{
			F.Dispose() ;
		}


		public bool EOF()
		{
			return sr.Peek() == -1 ;
		}


		public string FileName()
		{
			return _Name ;
		}


		public string ReadLn()
		{
			string S  = sr.ReadLine();
			return S ;
		}


		public void WriteLn( string Value )
		{
			sw.WriteLine(Value);
		}
	}


	public class PSBasic_File: TSBasic_File
	{
	}
}

