{$IFNDEF FPC}
  {$N+}
{$ENDIF}
{
              Program Name : Basic_Files
              Package Name : Subroutine Library - Layer 0
              Purpose      : Basic File classes
              Institution  : Conroy & Conroy Co.
              Date Written : 14-Feb-2021
              Written By   : Alan Conroy
              Version      : 2.0

              TO THE GLORY OF GOD THROUGH JESUS CHRIST

        Copyright (C) 2021 by Conroy & Conroy.  Released to the public domain.

        *********************************************************
        *                                                       *
        *        M O D I F I C A T I O N   H I S T O R Y        *
        *                                                       *
        *********************************************************

            DATE      BY             REASON

        *********************************************************
        *                                                       *
        *           P R O G R A M   P U R P O S E               *
        *                                                       *
        *********************************************************

        This unit has simplified base file functions and classes suitable for
        use in Subroutine Layer 0.
}

unit Basic_Files ;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

{$I EDEFINES.INC}

uses { Pascal... }
     sysutils, { FileExists }

     { Subroutine library... }
     _Common ; { TObject }

{ File open flags }
const FOF_Read_Write = 0 ; { Open for reading and writing }
const FOF_Read_Only = 1 ; { Open for reading only }
const FOF_Append = 2 ; { Open and position at end of file }
const FOF_Must_Exist = 4 ; { File must exist }
const FOF_Create = 8 ; { Create new file/overwrite old file }

const FErr_Not_Found = 1 ; { File doesn't exist }

type TSBasic_File = object( TObject )
                        private
                            F : text ;
                            _Name : string ;

                        public
                            function Open( const Name : string ;
                                Flags : integer ) : integer ; virtual ;
                            procedure Close() ; virtual ;
                            function EOF() : boolean ; virtual ;
                            function FileName : string ; virtual ;

                            function ReadLn() : string ; virtual ;
                            procedure WriteLn( const Value : string ) ; virtual ;
                    end ;

type PSBasic_File = ^TSBasic_File ;

function Open_Text_File( const Name : string ; Flags : integer ; var Err : integer  ) : PSBasic_File ;


implementation

function Open_Text_File( const Name : string ; Flags : integer ; var Err : integer  ) : PSBasic_File ;

var Fi : PSBasic_File ;

begin
    if( ( Flags and FOF_Must_Exist ) <> 0 ) then
    begin
        if( not FileExists( Name ) ) then
        begin
            Open_Text_File := nil ;
            Err := FErr_Not_Found ;
            exit ;
        end ;
    end ;

    Fi := new( PSBasic_File, Init() ) ;
    Err := Fi.Open( Name, Flags ) ;
    if( Err <> 0 ) then
    begin
        dispose( Fi, Done ) ;
        Open_Text_File := nil ;
        exit ;
    end ;
    Open_Text_File := Fi ;
end ;


{ TSBasic_File method... }

function TSBasic_File.Open( const Name : string ; Flags : integer ) : integer ;

var E, Saved : integer ;

begin
    if( ( Flags and FOF_Must_Exist ) <> 0 ) then
    begin
        if( not FileExists( Name ) ) then
        begin
            Open := FErr_Not_Found ;
            exit ;
        end ;
    end ;
    assignfile( F, Name ) ;
    Saved := FileMode ;
    case ( Flags and 1 ) of
        FOF_Read_Write : FileMode := fmOpenReadWrite ;
        FOF_Read_Only : FileMode := fmOpenRead ;
    end ;
    if( ( Flags and FOF_Append ) <> 0 ) then
    begin
        {$I-}
        append( F ) ;
        {$I+}
        E := IOResult ;
        if( E <> 0 ) then
        begin
            FileMode := Saved ;
            Open := E ;
            exit ;
        end ;
    end else
    if( ( Flags and FOF_Create ) <> 0 ) then
    begin
        {$I-}
        rewrite( F ) ;
        {$I+}
        E := IOResult ;
        if( E <> 0 ) then
        begin
            FileMode := Saved ;
            Open := E ;
            exit ;
        end ;
    end else
    begin
        {$I-}
        reset( F ) ;
        {$I+}
        E := IOResult ;
        if( E <> 0 ) then
        begin
            FileMode := Saved ;
            Open := E ;
            exit ;
        end ;
    end ;
    FileMode := Saved ;
    _Name := Name ;
    Open := 0 ;
end ;


procedure TSBasic_File.Close() ;

begin
    {$I-}
    system.close( F ) ;
    {$I+}
    IOResult ;
end ;


function TSBasic_File.EOF() : boolean ;

begin
    EOF := system.eof( F ) ;
end ;


function TSBasic_File.FileName : Ansistring ;

begin
    FileName := _Name ;
end ;


function TSBasic_File.ReadLn() : string ;

var S : string ;

begin
    system.readln( F, S ) ;
    Readln := S ;
end ;


procedure TSBasic_File.WriteLn( const Value : string ) ;

begin
    system.Writeln( F, Value ) ;
end ;



end.
