/*
        Program Name : Collect
        Package Name : Subroutine Library - Layer 0
        Purpose      : TCollection for Delphi
        Institution  : Conroy & Conroy Co.
        Date Written : 03-Jul-1996
	    Last Modified: 17-May-2002
        Written By   : Alan Conroy
        Version      : 2.0A

        Copyright 1996-2002 by Alan Conroy.  Released to the public domain.
        
        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY          REASON

          1-Nov-2002    EAC         Fix AtInsert.  Correct BP compilation error.

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

	  This unit provides the TCollection definition to reduce the
        differences between the various versions of Borland Pascal/Delphi.  This
        is not in Compatib beecause of unit dependancy issues.  It also provides
        some other TCollection variants.
*/

#include "Collect.hpp"

#include "CommBase.hpp"
#include <algorithm> // std::find
#ifdef CC_Regression_Test
  #include <iostream>
#endif

string EditUp( string S )
{
//	std::transform(S.begin(), S.end(), S.begin(), [](unsigned char &c){ return ::toupper(c); });
	std::transform(S.begin(), S.end(), S.begin(), ::toupper);
	return S ;
} ;


// TCollection methods...

// API...

void* TCollection::At( int Index )
{
    if( ( Index < 0 ) || ( Index > List.size() - 1 ) )
    {
        return NULL ;
    } 
	else
    {
        return List[ Index ] ;
    } ;
} ;


void TCollection::AtPut( int Index, void* Value )
{
    List[ Index ] = Value ;      
} ;


int TCollection::Count()
{
    return List.size() ;
} ;


void TCollection::Delete( void* Item )
{
    for(int i=0;i<List.size();i++)
	{
		if(List[i]==Item)
		{
			List.erase(List.begin() + i) ;
		}
	}	
} ;


void TCollection::AtDelete( int Item )
{
    List.erase( List.begin() + Item ) ;
} ;


int TCollection::IndexOf( void* Item )
{
	std::vector<void*>::iterator it;

	it = std::find(List.begin(), List.end(), Item);
    if(it != List.end())
    {
        return distance(List.begin(), it);
    }
    return -1;
} ;


void TCollection::Insert( void* Item )
{
    List.push_back( Item ) ;
} ;


void TCollection::DeleteAll()
{
    while( Count() > 0 )
    {
        AtDelete( Count() - 1 ) ;
    } ;
} ;


void TCollection::AtInsert( int Index, void* Item )
{
    List.insert( List.begin() + Index, Item ) ;
} ;


void TCollection::SetLimit( int ALimit )
{
    List.reserve( ALimit ) ;
} ;


int TCollection::Limit()
{
    return List.capacity() ;
} ;


  #ifdef CC_Regression_Test
void TCollection::Test()
{
    int Loop ;
	
	if( Count() != 0 )
    {
        cout << "Count <> 0" ;
        exit(0) ;
    } ;
    if( IndexOf( (void*) 1 ) != -1 )
    {
        cout << "IndexOf is invalid" ;
        exit(0) ;
    } ;
    for( Loop = 0; Loop <= 5; Loop ++ )
    {
        Insert( (void*)( Loop * 2 ) ) ;
    } ;
    if( Count() != 6 )
    {
        cout << "Count <> 6" ;
        exit(0) ;
    } ;
    for( Loop = 0; Loop <= 5; Loop ++ )
    {
        if( At( Loop ) != (void*)( Loop * 2 ) )
        {
            cout << "At returned wrong value" ;
            exit(0) ;
        } ;
    } ;
    for( Loop = 0; Loop <= 5; Loop ++ )
    {
        AtPut( Loop, (void*)( Loop * 2 + 1 ) ) ;
    } ;
    for( Loop = 0; Loop <= 5; Loop ++ )
    {
        if( At( Loop ) != (void*)( Loop * 2 + 1 ) )
        {
            cout << "AtPut incorrect" ;
            exit(0) ;
        } ;
    } ;
    if( IndexOf( (void*) 3 ) != 1 )
    {
        cout << "Indexof incorrect" ;
        exit(0) ;
    } ;
    Delete( (void*) 3 ) ;
    if( IndexOf( (void*) 3 ) != -1 )
    {
        cout << "Delete malfunctioned" ;
        exit(0) ;
    } ;
    for( Loop = Count() - 1; Loop >= 0; Loop-- )
    {
        AtDelete( Loop ) ;
    } ;
    if( Count() != 0 )
    {
        cout << "Count <> 0" ;
        exit(0) ;
    } ;
    if( At( -1 ) != NULL )
    {
        cout << "At( -1 ) did not return 0" ;
        exit(0) ;
    } ;
} ;
  #endif


// TString_Collection methods...

string TString_Collection::At( int Index )
{
    if( ( Index < 0 ) || ( Index > List.size() - 1 ) )
    {
        string S = "";
		return S;
	}
    else
    {
        return List[ Index ] ;
    } ;
} ;


void TString_Collection::AtPut( int Index, string Value )
{
    List[ Index ] = Value ;
} ;


int TString_Collection::Count()
{
    return List.size() ;
} ;


void TString_Collection::Delete( string Item )
{
    AtDelete( IndexOf( Item ) ) ;
} ;


void TString_Collection::AtDelete( int Item )
{
    List.erase( List.begin() + Item ) ;
} ;


int TString_Collection::IndexOf( string Item )
{
	std::vector<string>::iterator it;

	it = std::find(List.begin(), List.end(), Item);
    if(it != List.end())
    {
        return distance(List.begin(), it);
    }
    return -1;
} ;


int TString_Collection::Index_Of_Uppercase( string Item )
{
    int Loop;
	
	Item = EditUp( Item ) ;
    for( Loop = 0; Loop < Count() ; Loop++ )
    {
        if( Item == EditUp( At( Loop ) ) )
        {
            return Loop ;
        } ;
    } ;
    return -1 ;
} ;


void TString_Collection::Insert( string Item )
{
    List.push_back( Item ) ;
} ;


void TString_Collection::DeleteAll()
{
    while( Count() > 0 )
    {
        AtDelete( Count() - 1 ) ;
    } ;
} ;


void TString_Collection::AtInsert( int Index, string Item )
{
    List.insert( List.begin() + Index, Item ) ;
} ;


void TString_Collection::SetLimit( int ALimit )
{
    List.reserve( ALimit ) ;
} ;


int TString_Collection::Limit()
{
    return List.capacity() ;
} ;



// TInteger_Collection methods...

int TInteger_Collection::At( int Index )
{
    if( ( Index < 0 ) || ( Index > Count() - 1 ) ) 
    {
        return 0 ;
    } 
	return List[ Index ] ;
} ;


void TInteger_Collection::AtPut( int Index, int Value )
{
    List[ Index ] = Value ;
} ;


int TInteger_Collection::Count()
{
    return List.size() ;
} ;


void TInteger_Collection::Delete( int Item )
{
    AtDelete( IndexOf( Item ) ) ;
} ;


void TInteger_Collection::AtDelete( int Item )
{
    List.erase( List.begin() + Item ) ;
} ;


int TInteger_Collection::IndexOf( int Item )
{
	std::vector<int>::iterator it;

	it = std::find(List.begin(), List.end(), Item);
    if(it != List.end())
    {
        return distance(List.begin(), it);
    }
    return -1;
} ;


void TInteger_Collection::Insert( int Item )
{
    List.push_back( Item ) ;
} ;


void TInteger_Collection::DeleteAll()
{
    while( Count() > 0 )
    {
        AtDelete( Count() - 1 ) ;
    } ;
} ;


void TInteger_Collection::AtInsert( int Index, int Item )
{
    List.insert( List.begin() + Index, Item ) ;
} ;


void TInteger_Collection::SetLimit( int ALimit )
{
    List.reserve( ALimit ) ; 
} ;


int TInteger_Collection::Limit()
{
    return List.capacity() ;
} ;


#ifdef CC_Regression_Test
void TString_Collection::Test()
{
    int Loop ;

    if( Count() != 0 )
    {
        cout << "Count <> 0" ;
        exit(0) ;
    } ;
    if( IndexOf( "1" ) != -1 )
    {
        cout << "IndexOf is invalid" ;
        exit(0) ;
    } ;
    for( Loop = 0; Loop <= 5; Loop ++ )
    {
        Insert( to_string( Loop * 2 ) ) ;
    } ;
    if( Count() != 6 )
    {
        cout << "Count <> 6" ;
        exit(0) ;
    } ;
    for( Loop = 0; Loop <= 5; Loop ++ )
    {
        if( At( Loop ) != to_string( Loop * 2 ) )
        {
            cout << "At returned wrong value" ;
            exit(0) ;
        } ;
    } ;
    for( Loop = 0; Loop <= 5; Loop ++ )
    {
        AtPut( Loop, to_string( Loop * 2 + 1 ) ) ;
    } ;
    for( Loop = 0; Loop <= 5; Loop ++ )
    {
        if( At( Loop ) != to_string( Loop * 2 + 1 ) )
        {
            cout << "AtPut incorrect" ;
            exit(0) ;
        } ;
    } ;
    if( IndexOf( "3" ) != 1 )
    {
        cout << "Indexof incorrect" ;
        exit(0) ;
    } ;
    Delete( "3" ) ;
    if( IndexOf( "3" ) != -1 )
    {
        cout << "Delete malfunctioned" ;
        exit(0) ;
    } ;
    for( Loop = Count() - 1; Loop >= 0; Loop-- )
    {
        AtDelete( Loop ) ;
    } ;
    if( Count() != 0 )
    {
        cout << "Count <> 0" ;
        exit(0) ;
    } ;
    if( At( -1 ) != "" )
    {
        cout << "At( -1 ) did not return null string" ;
        exit(0) ;
    } ;
} ;


void TInteger_Collection::Test()
{
    int Loop ;

	if( Count() != 0 )
    {
        cout << "Count <> 0" ;
        exit(0) ;
    } ;
    if( IndexOf( 1 ) != -1 )
    {
        cout << "IndexOf is invalid" ;
        exit(0) ;
    } ;
    for( Loop = 0; Loop <= 5; Loop ++ )
    {
        Insert( Loop * 2 ) ;
    } ;
    if( Count() != 6 )
    {
        cout << "Count <> 6" ;
        exit(0) ;
    } ;
    for( Loop = 0; Loop <= 5; Loop++ )
    {
        if( At( Loop ) != Loop * 2 )
        {
            cout << "At returned wrong value" ;
            exit(0) ;
        } ;
    } ;
    for( Loop = 0; Loop <= 5; Loop++ )
    {
        AtPut( Loop, Loop * 2 + 1 ) ;
    } ;
    for( Loop = 0; Loop <= 5; Loop++ )
    {
        if( At( Loop ) != Loop * 2 + 1 )
        {
            cout << "AtPut incorrect" ;
            exit(0) ;
        } ;
    } ;
    if( IndexOf( 3 ) != 1 )
    {
        cout << "Indexof incorrect" ;
        exit(0) ;
    } ;
    Delete( 3 ) ;
    if( IndexOf( 3 ) != -1 )
    {
        cout << "Delete malfunctioned" ;
        exit(0) ;
    } ;
    for( Loop = Count() - 1; Loop>=0; Loop-- )
    {
        AtDelete( Loop ) ;
    } ;
    if( Count() != 0 )
    {
        cout << "Count <> 0" ;
        exit(0) ;
    } ;
    if( At( -1 ) != 0 )
    {
        cout << "At( -1 ) did not return 0" ;
        exit(0) ;
    } ;
} ;


void Test_Unit()
{
    TCollection TC ;
    TString_Collection TSC ;
    TInteger_Collection TIC ;

    TC.Test() ;
    TSC.Test() ;
    TIC.Test() ;
} ;

#endif
