/*
        Program Name : CommBase
        Package Name : Subroutine Library - Layer 0
        Purpose      : Common base objects (TBase_Object and TBase_Delphi_Object)
        Institution  : Conroy & Conroy Co.
        Date Written : 19-Mar-2002
        Written By   : Alan Conroy
        Version      : 1.0

		Copyright (C) 2002-2021 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

		*************************************************************
		*                                                           *
		*        M O D I F I C A T I O N   H I S T O R Y            *
		*                                                           *
		*************************************************************

			DATE        BY          REASON

		*************************************************************
		*                                                           *
		*        P R O G R A M   P U R P O S E                      *
		*                                                           *
		*************************************************************
*/

#include "CommBase.hpp"

using namespace std;

// Subroutine Library...
#ifdef CC_Track_Resources
	#include "ResMon.hpp"
#endif
#include "_ObjectF.hpp"
#include "L10n.hpp"

/* TBase_Delphi_Object methods... */

/* Constructors and destructors... */

TBase_Delphi_Object::TBase_Delphi_Object() : TDelphi_Object()
{
#ifdef CC_Track_Resources
	Add_Resource( RT_Object, (int)( *this ) ) ;
#endif

#ifdef CC_Debug
	Sanity = "TBase_Delphi_Object" ;
#endif
} ;


TBase_Delphi_Object::~TBase_Delphi_Object()
{
#ifdef CC_Debug
    Sanity = "                   " ;
#endif
    if( _Properties != NULL )
	{
		_Properties->Detach() ;
    } ;
#ifdef CC_Track_Resources
    Remove_Resource( RT_Object, integer( self ) ) ;
#endif
} ;


/* API... */

void stdcall TBase_Delphi_Object::Attach()
{
	_Reference_Count++ ;
} ;


bool stdcall TBase_Delphi_Object::Detach()
{
	_Reference_Count-- ;
    if( _Reference_Count <= 0 )
    {
		delete this ;
		return true ;
	} ;
	return false ;
} ;


int TBase_Delphi_Object::Reference_Count()
{
	return _Reference_Count ;
} ;


bool stdcall TBase_Delphi_Object::Is_Class( const string Name )
{
	try
	{
		return ( Name == "TBase_Delphi_Object" ) ;
	}
	catch(...)
	{
		return false ;
    } ;
} ;


TDelphi_Object* TBase_Delphi_Object::Clone()
{
	return NULL ;
} ;


TDynamic_Properties* TBase_Delphi_Object::Properties()
{
    if( _Properties == NULL )
    {
		_Properties = Object_Factory->Dynamic_Properties() ;
    } ;
	return _Properties ;
} ;


void TBase_Delphi_Object::Test()
{
	int I = Reference_Count() ;
	Attach() ;
	if( I + 1 != Reference_Count() )
    {
		cout << Text_Error_Attach_Failed ;
    } ;
    if( I == 0 ) /* No previous attachments */
    {
		Attach() ;
		I++ ;
    } ;
	Detach() ;
	if( I != Reference_Count() )
    {
		cout << Text_Error_Detach_Failed ;
	} ;
} ;


TDebug_Interface* TBase_Delphi_Object::Debugger()
{
	return NULL ;
} ;


bool TBase_Delphi_Object::Deserialize( TStream* Stream, int32 Options )
{
	return false ;
} ;


bool TBase_Delphi_Object::Persist( TStream* Stream )
{
	return false ;
} ;


bool TBase_Delphi_Object::Serialize( TStream* Stream, int32 Options )
{
	return false ;
} ;


bool TBase_Delphi_Object::Load( TStream* Stream )
{
	return false ;
} ;


TUnified_Exception* TBase_Delphi_Object::Set_Last_Error( TUnified_Exception* X )
{
	if( X == _Last_Error )
    {
		return X ;
    } ;
    if( _Last_Error != NULL )
    {
		_Last_Error->Detach() ;
    } ;
	_Last_Error = X ;
    if( _Last_Error != NULL )
    {
		_Last_Error->Attach() ;
	} ;
	return X ;
} ;


TUnified_Exception* TBase_Delphi_Object::Last_Error()
{
	return _Last_Error ;
} ;



/* TObject methods... */

TBase_Object::TBase_Object() : TObject()
{
#ifdef CC_Track_Resources
    Add_Resource( RT_Object, integer( @self ) ) ;
#endif

    _Reference_Count = 0 ;
	_Memory_Type = MT_Unknown ;
    _Properties = NULL ;
#ifdef CC_Debug
    Base_Object_Sanity = "TBase_Object" ;
#endif
} ;


TBase_Object::~TBase_Object()
{
#ifdef CC_Debug
    Base_Object_Sanity = "            " ;
#endif
    if( _Properties != NULL )
    {
		_Properties->Detach() ;
    } ;
#ifdef CC_Track_Resources
	Add_Resource( RT_Object, (int)(*this) ) ;
#endif
} ;


void TBase_Object::Free()
{
	if( this != NULL )
	{
		if( _Memory_Type == MT_Heap )
		{
			delete this ;
		} ;
	} ;
} ;


void TBase_Object::Attach()
{
#ifdef CC_Debug
    if( Base_Object_Sanity != "TBase_Object" )
    {
		cout << "Invalid object" ;
    } ;
#endif
	_Reference_Count++ ;
} ;


bool TBase_Object::Detach()
{
#ifdef CC_Debug
	if( Base_Object_Sanity != "TBase_Object" )
	{
		cout << "Invalid object" ;
	} ;
#endif
	_Reference_Count-- ;
	if( _Reference_Count <= 0 )
	{
#ifdef CC_Debug
		if( _Reference_Count < 0 )
		{
			cout << "Overly detached object" ;
		} ;
#endif
		Free() ;
		return true ;
	} ;
	return false ;
} ;


int TBase_Object::Reference_Count()
{
#ifdef CC_Debug
    if( Base_Object_Sanity != "TBase_Object" )
    {
		cout << "Invalid object" ;
    } ;
#endif
	return _Reference_Count ;
} ;


PBase_Object TBase_Object::Clone()
{
#ifdef CC_Debug
    if( Base_Object_Sanity != "TBase_Object" )
    {
		cout << "Invalid object" ;
    } ;
#endif
	return NULL ;
} ;


PSDynamic_Properties TBase_Object::Properties()
{
#ifdef CC_Debug
    if( Base_Object_Sanity != "TBase_Object" )
    {
		cout << "Invalid object" ;
    } ;
#endif
    if( _Properties == NULL )
    {
		_Properties = Object_Factory->Static_Dynamic_Properties() ;
    } ;
	return _Properties ;
} ;


string TBase_Object::Classname()
{
#ifdef CC_Debug
    if( Base_Object_Sanity != "TBase_Object" )
	{
		cout << "Invalid object" ;
    } ;
#endif
	return "TBase_Object" ;
} ;


bool TBase_Object::Is_Class( const string Name )
{
#ifdef CC_Debug
    if( Base_Object_Sanity != "TBase_Object" )
	{
		cout << "Invalid object" ;
    } ;
#endif
	if( Name == Classname() )
    {
		return true ;
	}
	return TObject::Is_Class( Name ) ;
} ;


void TBase_Object::Set_Memory_Type( TMemory_Type MT )
{
#ifdef CC_Debug
    if( Base_Object_Sanity != "TBase_Object" )
    {
		cout << "Invalid object" ;
    } ;
#endif
    _Memory_Type = MT ;
} ;


TMemory_Type TBase_Object::Get_Memory_Type()
{
#ifdef CC_Debug
    if( Base_Object_Sanity != "TBase_Object" )
    {
		cout << "Invalid object" ;
	} ;
#endif
	return _Memory_Type ;
} ;


bool TBase_Object::Deserialize( PSStream Stream, int32 Options )
{
#ifdef CC_Debug
    if( Base_Object_Sanity != "TBase_Object" )
    {
		cout << "Invalid object" ;
    } ;
#endif
	return false ;
} ;


bool TBase_Object::Persist( PSStream Stream )
{
#ifdef CC_Debug
    if( Base_Object_Sanity != "TBase_Object" )
	{
		cout << "Invalid object" ;
	} ;
#endif
	return false ;
} ;


bool TBase_Object::Serialize( PSStream Stream, int32 Options )
{
#ifdef CC_Debug
    if( Base_Object_Sanity != "TBase_Object" )
    {
		cout << "Invalid object" ;
    } ;
#endif
	return false ;
} ;


bool TBase_Object::Load( PSStream Stream )
{
#ifdef CC_Debug
    if( Base_Object_Sanity != "TBase_Object" )
    {
        cout << "Invalid object" ) ;
    } ;
#endif
	return false ;
} ;


void TBase_Object::Test()
{
#ifdef CC_Debug
    if( Base_Object_Sanity != "TBase_Object" )
    {
		cout << "Invalid object" ;
    } ;
#endif
	int I = Reference_Count() ;
	Attach() ;
    if( I + 1 != Reference_Count() )
	{
		cout << Text_Error_Attach_Failed ;
    } ;
	if( I == 0 ) /* No previous attachments */
    {
		Attach() ;
		I++ ;
    } ;
	Detach() ;
	if( I != Reference_Count() )
    {
		cout << Text_Error_Detach_Failed ;
    } ;
} ;


PSDebug_Interface TBase_Object::Debugger()
{
#ifdef CC_Debug
	if( Base_Object_Sanity != "TBase_Object" )
	{
		cout << "Invalid object" ;
	} ;
#endif
	return NULL ;
} ;


PSUnified_Exception TBase_Object::Set_Last_Error( PSUnified_Exception X )
{
#ifdef CC_Debug
    if( Base_Object_Sanity != "TBase_Object" )
    {
		cout << "Invalid object" ;
    } ;
#endif
	return X ;
} ;


PSUnified_Exception TBase_Object::Last_Error()
{
#ifdef CC_Debug
	if( Base_Object_Sanity != "TBase_Object" )
    {
		cout << "Invalid object" ;
    } ;
#endif
	return _Last_Error ;
} ;


#ifdef CC_Regression_Test
void Test_Unit()
{
	TBase_Delphi_Object TBDO = new TBase_Delphi_Object ;
	TBDO->Test() ;
	TBDO->Free() ;

	TBase_Object TBO ;
	TBO.Test() ;
} ;
#endif

