/*
		Program Name : CommBase
		Package Name : Subroutine Library - Layer 0
		Purpose      : Common base objects (TBase_Object and TBase_Delphi_Object)
		Institution  : Conroy & Conroy Co.
		Date Written : 19-Mar-2002
		Written By   : Alan Conroy
		Version      : 1.0

		Copyright (C) 2002-2021 by Alan Conroy.  Released to the public domain.

		TO THE GLORY OF GOD THROUGH JESUS CHRIST

		*************************************************************
		*                                                           *
		*        M O D I F I C A T I O N   H I S T O R Y            *
		*                                                           *
		*************************************************************

			DATE        BY          REASON

		*************************************************************
		*                                                           *
		*        P R O G R A M   P U R P O S E                      *
		*                                                           *
		*************************************************************
*/

using System;
using _Common_unit; // TObject
using _DebugIn_unit; // TDebug_Interface
using _Propert_unit; // TProperties
using _Streams_unit; // TSStream
using _UE_unit; // tUnified_Exception
using _ObjectF_unit; // Object_Factory
using L10n_unit; // Text_*

namespace CommBase_unit
{
	public class CommBase
	{
		// Serialization/deserialization options...
		public const int SDO_Visual = 1 ; // Visual aspects
		public const int SDO_Deep = 2 ; // Include serialization of embedded/reference objects
		//-32768 = reserved for future use
		public const int SDO_User = 65536 ; // This flag (and following) reserved for the user
	}

	public class TBase_Delphi_Object : TObject
	{
		// Instance data...
		private int _Reference_Count = 0 ;
		private TDynamic_Properties _Properties = null ;
		private TUnified_Exception _Last_Error = null ;

		// API...
		public override void Attach()
		{
			_Reference_Count++ ;
		}


		public override bool Detach()
		{
			_Reference_Count-- ;
			return ( _Reference_Count == 0 ) ;
		}


		public virtual int Reference_Count()
		{
			return _Reference_Count ;
		}


		public override bool Is_Class( string Name )
		{
			return ( Name == Classname() ) ;
		}


		public virtual TDelphi_Object Clone()
		{
			return null ;
		}


		public virtual TDynamic_Properties Properties()
		{
			if( _Properties == null )
			{
				_Properties = new TSObject_Factory().Dynamic_Properties() ;
			}
			return _Properties ;
		}


		public virtual TDebug_Interface Debugger()
		{
			return null ;
		}


		public virtual bool Deserialize( TStream Stream, int Options )
		{
			return false ;
		}


		public virtual bool Persist( TStream Stream )
		{
			return false ;
		}


		public virtual bool Serialize( TStream Stream, int Options )
		{
			return false ;
		}


		public virtual bool Load( TStream Stream )
		{
			return false ;
		}


		public virtual TUnified_Exception Set_Last_Error( TUnified_Exception X )
		{
			_Last_Error = X ;
			return X ;
		}


		public virtual TUnified_Exception Last_Error()
		{
			return _Last_Error ;
		}


		public virtual void Free()
		{
		}


		public override string Classname()
		{
			return "TBase_Delphi_Object" ;
		}
	} ; // TBase_Delphi_Object


	public class TBase_Object : TBase_Delphi_Object
	{
	} ;

	
	public class PBase_Object : TBase_Object
	{
	} ;
}
