/*
        Program Name : CommBase
        Package Name : Subroutine Library - Layer 0
        Purpose      : Common base objects (TBase_Object and TBase_Delphi_Object)
        Institution  : Conroy & Conroy Co.
        Date Written : 19-Mar-2002
        Written By   : Alan Conroy
        Version      : 1.0

		Copyright (C) 2002-2021 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY          REASON

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
		*************************************************************
*/

#ifndef COMMBASE_HPP
	#define COMMBASE_HPP

#include "eDefines.hpp"

// C&C...
#include "_Common.hpp"
#include "_DebugIn.hpp"
#include "_Propert.hpp"
#include "_Streams.hpp"
#include "_UE.hpp"
#include <string>
#include <iostream>

using namespace std;

// Serialization/deserialization options...
const SDO_Visual = 1 ; // Visual aspects
const SDO_Deep = 2 ; // Include serialization of embedded/reference objects
// 4-32768 = reserved for future use
const SDO_User = 65536 ; // This flag (and following) reserved for the user

class TBase_Delphi_Object: public TDelphi_Object
{
	private: /* Instance data... */
		int _Reference_Count ;
		TDynamic_Properties* _Properties ;
	#ifdef CC_Debug
		string Sanity ;
	#endif
		TUnified_Exception* _Last_Error ;

	public: /* Constructors and destructors... */
		TBase_Delphi_Object() ;
		virtual ~TBase_Delphi_Object() ;

	public: /* API... */
		virtual void stdcall Attach() ;
		virtual bool stdcall Detach() ;
		virtual bool stdcall Is_Class( const string Name ) ;

	protected: /* Internal utility */
		virtual TUnified_Exception* Set_Last_Error( TUnified_Exception* X ) ;

	public: /* API... */
		int Reference_Count() ;

		virtual TDelphi_Object* Clone() ;

		virtual TDynamic_Properties* Properties() ;
		virtual TDebug_Interface* Debugger() ;
		virtual void Test() ;

		virtual bool Deserialize( TStream* Stream,
			int32 Options ) ;
		virtual bool Serialize( TStream* Stream,
			int32 Options ) ;
		virtual bool Load( TStream* Stream ) ;
		virtual bool Persist( TStream* Stream ) ;
		virtual TUnified_Exception* Last_Error() ;
} ; // TBase_Delphi_Object


class TBase_Object: public TObject
{
	private: /* Instance data... */
		TMemory_Type _Memory_Type ;
		int32 _Reference_Count ;
		PSDynamic_Properties _Properties ;
#ifdef CC_Debug
		string Base_Object_Sanity ;
#endif
	protected:
		PSUnified_Exception _Last_Error ;

	public: /* Constructors and destructors... */
		TBase_Object() ;
		virtual ~TBase_Object() ;
		void Free() ;

	protected: /* Internal... */
		virtual PSUnified_Exception Set_Last_Error( PSUnified_Exception X ) ;

	public: /* API... */
		virtual void Attach() ;
		virtual bool Detach() ;
		virtual string Classname() ;
		virtual bool Is_Class( const string Name ) ;
		void Set_Memory_Type( TMemory_Type MT ) ;
		virtual TMemory_Type Get_Memory_Type() ;
		int Reference_Count() ;

		virtual TBase_Object* Clone() ;
		virtual PSDynamic_Properties Properties() ;
		virtual PSDebug_Interface Debugger() ;
		virtual void Test() ;

		virtual bool Deserialize( PSStream Stream, int32 Options ) ;
		virtual bool Serialize( PSStream Stream, int32 Options ) ;
		virtual bool Load( PSStream Stream ) ;
		virtual bool Persist( PSStream Stream ) ;
		virtual PSUnified_Exception Last_Error() ;
} ; // TBase_Object

#define PBase_Object TBase_Object*

#endif

