// Serialization/deserialization options...
const SDO_Visual = 1 ; // Visual aspects
const SDO_Deep = 2 ; // Include serialization of embedded/reference objects
// 4-32768 = reserved for future use
const SDO_User = 65536 ; // This flag (and following) reserved for the user

class TBase_Object extends TDelphi_Object
{
	Sanity = "";

	constructor()
	{
		super();
		if( typeof CC_Debug !== 'undefined' )	
		{
		    this.Sanity = "TBase_Delphi_Object" ;
		}
	}


	// API...

	Attach()
	{
    	this._Reference_Count++ ;
	}


	Detach()
	{
    	this._Reference_Count-- ;
    	return ( this._Reference_Count == 0 ) ;
	}


	Reference_Count()
	{
    	return this._Reference_Count ;
	}


	Is_Class( Name )
	{
    	if( Name == this.Classname() )
    	{
    	    return true ;
    	}
   	    return super.Is_Class( Name ) ;
	}


	_Clone()
	{
	    return JSON.parse(JSON.stringify(this)) ;
	}


	Properties()
	{
    	if( this._Properties == null )
    	{
    	    this._Properties = Object_Factory.Dynamic_Properties() ;
    	} ;
    	return _Properties ;
	}


	Test()
	{
    	I = this.Reference_Count() ;
    	this.Attach() ;
    	if( I + 1 != this.Reference_Count() )
    	{
    	    alert( "Attach failed" ) ;
    	} ;
    	if( I == 0 ) // No previous attachments
    	{
    	    this.Attach() ;
    	    I++ ;
    	} ;
    	this.Detach() ;
    	if( I != this.Reference_Count() )
    	{
    	    alert( "Detach failed" ) ;
    	} ;
	}


	Debugger()
	{
	    return null ;
	}


	Deserialize( Stream )
	{
    	return false ;
	}


	Persist( Stream )
	{
    	return false ;
	}


	Serialize( Stream )
	{
    	return false ;
	}


	Load( Stream )
	{
    	return false ;
	}
} ; // TBase_Object


class TBase_Delphi_Object extends TBase_Object
{
	_Memory_Type = MT_Unknown ;
    _Properties = null ;
    _Reference_Count = 0 ;
    Base_Object_Sanity = "" ;
    Local_Serial_Number = 0 ;

	constructor()
	{
		super();
		if( typeof CC_Debug !== 'undefined' )	
		{
		    this.Base_Object_Sanity = "TBase_Object" ;
		}
	}


	Free()
	{
		delete this;
	}


	Attach()
	{
		if( typeof CC_Debug !== 'undefined' )	
		{
		    if( this.Base_Object_Sanity != "TBase_Object" )
		    {
		        alert( "Invalid object" ) ;
		    } ;
		} ;
	    this._Reference_Count++ ;
	}


	Detach()
	{
		if( typeof CC_Debug !== 'undefined' )	
		{
		    if( this.Base_Object_Sanity != "TBase_Object" )
		    {
		        alert( "Invalid object" ) ;
		    } ;
		} ;
	    this._Reference_Count-- ;
	    if( this._Reference_Count <= 0 )
	    {
			if( typeof CC_Debug !== 'undefined' )	
			{
			        if( this._Reference_Count < 0 )
			        {
			            alert( "Overly detached object" ) ;
			        } ;
			} ;
	    } ;
	}


	Reference_Count()
	{
		if( typeof CC_Debug !== 'undefined' )	
		{
		    if( this.Base_Object_Sanity != "TBase_Object" )
		    {
		        alert( "Invalid object" ) ;
		    } ;
		} ;
	    return this._Reference_Count ;
	}


	_Clone()
	{
		if( typeof CC_Debug !== 'undefined' )	
		{
		    if( this.Base_Object_Sanity != "TBase_Object" )
		    {
		        alert( "Invalid object" ) ;
		    } ;
		} ;
	    return JSON.parse(JSON.stringify(this)) ;
	}


	Properties()
	{
		if( typeof CC_Debug !== 'undefined' )	
		{
		    if( this.Base_Object_Sanity != "TBase_Object" )
		    {
		        alert( "Invalid object" ) ;
		    } ;
		}
	    if( this._Properties == null )
	    {
	        this._Properties = Object_Factory.Static_Dynamic_Properties() ;
	    } ;
	    return this._Properties ;
	}


	Classname()
	{
		return this.constructor.name;
	}


	Set_Memory_Type( MT )
	{
		if( typeof CC_Debug !== 'undefined' )	
		{
		    if( this.Base_Object_Sanity != "TBase_Object" )
		    {
		        alert( "Invalid object" ) ;
		    } ;
		}
	    this._Memory_Type = MT ;
	}


	Get_Memory_Type()
	{
		if( typeof CC_Debug !== 'undefined' )	
		{
		    if( this.Base_Object_Sanity != "TBase_Object" )
		    {
		        alert( "Invalid object" ) ;
		    } ;
		}
	    return this._Memory_Type ;
	}


	Deserialize( Stream )
	{
		if( typeof CC_Debug !== 'undefined' )	
		{
		    if( this.Base_Object_Sanity != "TBase_Object" )
		    {
		        alert( "Invalid object" ) ;
		    } ;
		}
	    return false ;
	}


	Persist( Stream )
	{
		if( typeof CC_Debug !== 'undefined' )	
		{
		    if( this.Base_Object_Sanity != "TBase_Object" )
		    {
		        alert( "Invalid object" ) ;
		    } ;
		}
	    return false ;
	}


	Serialize( Stream )
	{
		if( typeof CC_Debug !== 'undefined' )	
		{
		    if( this.Base_Object_Sanity != "TBase_Object" )
		    {
		        alert( "Invalid object" ) ;
		    } ;
		} ;
	    return false ;
	}


	Load( Stream )
	{
		if( typeof CC_Debug !== 'undefined' )	
		{
		    if( this.Base_Object_Sanity != "TBase_Object" )
		    {
		        alert( "Invalid object" ) ;
		    } ;
		} ;
	    return false ;
	}


	Test()
	{
		if( typeof CC_Debug !== 'undefined' )	
		{
		    if( this.Base_Object_Sanity != "TBase_Object" )
		    {
		        alert( "Invalid object" ) ;
		    } ;
		} ;
	    var I = this.Reference_Count() ;
	    this.Attach() ;
	    if( I + 1 != this.Reference_Count() )
	    {
	        alert( "Attach failed" ) ;
	    } ;
	    if( I == 0 ) // No previous attachments
	    {
	        this.Attach() ;
	        I++ ;
	    } ;
	    this.Detach() ;
	    if( I != this.Reference_Count() )
	    {
	        alert( "Detach failed" ) ;
	    } ;
	}


	Debugger()
	{
		if( typeof CC_Debug !== 'undefined' )	
		{
	    	if( Base_Object_Sanity != "TBase_Object" )
	    	{
	    	    alert( "Invalid object" ) ;
	    	} ;
		} ;
	    return null ;
	}
}; // TBase_Delphi_Object


class PBase_Object extends TBase_Object
{
} ;
