#        Program Name : Common
#        Package Name : Subroutine Library - Layer 0
#        Purpose      : Common base objects (TBase_Object and TBase_Delphi_Object)
#        Institution  : Conroy & Conroy Co.
#        Date Written : 19-Mar-2002
#        Written By   : Alan Conroy
#        Version      : 1.0
#
#        Copyright (C) 2002-2005 by Alan Conroy.  Released to the public domain.
#
#        TO THE GLORY OF GOD THROUGH JESUS CHRIST
#
#        *************************************************************
#        *                                                           *
#        *        M O D I F I C A T I O N   H I S T O R Y            *
#        *                                                           *
#        *************************************************************
#
#        *    DATE        BY          REASON                         *
#
#        *************************************************************
#        *                                                           *
#        *        P R O G R A M   P U R P O S E                      *
#        *                                                           *
#        *************************************************************

# C&C...
from _Common import *
from _DebugIn import *
from _Propert import *
from _Streams import *
from _ObjectF import *

class TBase_Delphi_Object( TDelphi_Object ):
    # Instance data...
    _Reference_Count = 0
    _Properties = None
    Sanity = ""

    # Constructors and destructors...
    def __init__(self):
        self.Sanity = "TBase_Delphi_Object"
        
    def __del__(self):
        self.Sanity = "                   "
        if( self._Properties != None ):
            self._Properties.Detach()


    # API...
    def Attach( self ):
        ++self._Reference_Count


    def Detach( self ):
        --self._Reference_Count
        if( self._Reference_Count <= 0 ):
            self.Free()
            return true
        return false


    def Reference_Count( self ):
        return self._Reference_Count


    def Classname( self ):
        return self.__class__.__name__
        
    def Is_Class( self, Name ):
        return ( Name == self.Classname() )


    def Clone( self ):
        return None


    def Properties( self ):
        if( self._Properties == None ):
            self._Properties = Object_Factory.Dynamic_Properties
        return self._Properties


    def Test( self ):
        I = 0

        I = self.Reference_Count()
        self.Attach()
        if( I + 1 != self.Reference_Count() ):
            print( "Attach failed" )
        if( I == 0 ): # No previous attachments
            self.Attach()
            ++I
        self.Detach() ;
        if( I != self.Reference_Count() ):
            print( "Detach failed" )


    def Debugger( self ):
        return None


    def Deserialize( self, Stream ):
        return False


    def Persist( self, Stream ):
        return False


    def Serialize( self, Stream ):
        return False


    def Load( self, Stream ):
        return False


class TBase_Object( TObject ):
    # Instance data...
    _Memory_Type = MT_Unknown
    _Reference_Count = 0
    _Properties = None
    Base_Object_Sanity = ""

    def __init__(self):
        self.Base_Object_Sanity = "TBase_Object"


    def __del__(self):
        Base_Object_Sanity = "            "
        if( self._Properties != None ):
            self._Properties.Detach()


    def Free(self):
        if( self != None ):
            del self


    def Attach(self):
        if( self.Base_Object_Sanity != "TBase_Object" ):
            print( "Invalid object" )
        self._Reference_Count = self._Reference_Count + 1


    def Detach( self ):
        if( self.Base_Object_Sanity != "TBase_Object" ):
            print( "Invalid object" ) ;
        self._Reference_Count = self._Reference_Count - 1
        if( self._Reference_Count <= 0 ):
            if( self._Reference_Count < 0 ):
                print( "Overly detached object" )
            del self


    def Reference_Count( self ):
        if( self.Base_Object_Sanity != "TBase_Object" ):
            print( "Invalid object" )
        return self._Reference_Count


    def Clone( self ):
        if( self.Base_Object_Sanity != "TBase_Object" ):
            print( "Invalid object" )
        return None


    def Properties( self ):
        if( self.Base_Object_Sanity != "TBase_Object" ):
            print( "Invalid object" )
        if( self._Properties == None ):
            self._Properties = Object_Factory.Static_Dynamic_Properties()
        return self._Properties


    def Is_Class( self, Name ):
        if( self.Base_Object_Sanity != "TBase_Object" ):
            print( "Invalid object" )
        if( Name == self.Classname() ):
            return True
        else:
            return super().Is_Class( Name )


    def Set_Memory_Type( self, MT ):
        if( self.Base_Object_Sanity != "TBase_Object" ):
            print( "Invalid object" ) ;
        self._Memory_Type = MT ;


    def Get_Memory_Type( self ):
        if( self.Base_Object_Sanity != "TBase_Object" ):
            print( "Invalid object" ) ;
        return self._Memory_Type ;


    def Deserialize( self, Stream ):
        if( self.Base_Object_Sanity != "TBase_Object" ):
            print( "Invalid object" )
        return False


    def Persist( self, Stream ):
        if( self.Base_Object_Sanity != "TBase_Object" ):
            print( "Invalid object" )
        return False


    def Serialize( self, Stream ):
         if( self.Base_Object_Sanity != "TBase_Object" ):
            print( "Invalid object" )
         return False


    def Load( self, Stream ):
        if( self.Base_Object_Sanity != "TBase_Object" ):
            print( "Invalid object" )
        return False


    def Test( self ):
        if( self.Base_Object_Sanity != "TBase_Object" ):
            print( "Invalid object" )
        I = self.Reference_Count()
        self.Attach()
        if( I + 1 != self.Reference_Count() ):
            print( "Attach failed" )
        if( I == 0 ): # No previous attachments
            self.Attach()
            I = I + 1
        self.Detach()
        if( I != self.Reference_Count() ):
            print( "Detach failed" )


    def Debugger( self ):
        if( self.Base_Object_Sanity != "TBase_Object" ):
            print( "Invalid object" )
        return None


class PBase_Object( TBase_Object ):
    pass
