/*
		Program Name : _Cache
		Package Name : Subroutine Library - Layer 1
		Purpose      : Cache class definition
		Institution  : Conroy & Conroy Co.
		Date Written : 18-Jun-1994
		Written By   : Alan Conroy
		Version      : 1.0

		Copyright (C) 1994-2021 by Alan Conroy.  Released to the public domain.

		TO THE GLORY OF GOD THROUGH JESUS CHRIST

		*************************************************************
		*                                                           *
		*        M O D I F I C A T I O N   H I S T O R Y            *
		*                                                           *
		*************************************************************

			DATE        BY          REASON

		*************************************************************
		*                                                           *
		*        P R O G R A M   P U R P O S E                      *
		*                                                           *
		*************************************************************

		  This unit defines classes for handling caches.
*/

using System;
using _Common_unit ;
using CommBase_unit ; // TBase_Object

namespace _Cache_unit
{
	public class TCache_Statistics
	{
		public long Hits ; // Total read hits
		public long Misses ; // Read misses
		// Note: Hits + Misses = total reads
		public long Writes ; // Total writes
		public long Turns ; // Number of times blocks were thrown away to make room for more
		public long Replacements ; // Number of write hits
		// Note: Write - Replacements = non-replacement writes
	} ;

	public class TCache : TBase_Object
	{
		protected TCache_Statistics Statistics ;

		// API...
		public virtual int Read_Cache( string Data,
		   long Address, long Size )
		/* Reads up to Size bytes from the cache for the specified
		 address.  Returns the number of bytes actually read (if
		 0 then the data was not cached).  Cached data is copied
		 to Data. */
		{
			_Common C = new _Common();
			C.Abstract_Error() ;
			return 0 ;
		}

		public virtual void Write_Cache( string Data,
		   long Address, long Size )
		// Writes to cache up to Size bytes from Data.
		{
			_Common C = new _Common();
			C.Abstract_Error() ;
		}

		public virtual void Clear() // Clears cache
		{
			_Common C = new _Common();
			C.Abstract_Error() ;
		}

		public virtual void Initialize() // Initialize object
		{
			_Common C = new _Common();
			C.Abstract_Error() ;
		}

		public virtual void Terminate() // Terminate object
		{
			_Common C = new _Common();
			C.Abstract_Error() ;
		}

		public virtual void Set_Max_Size( long Size )
		/* Sets maximum cache size in bytes.  If this is more
		 than is currently cached, some cache is thrown away. */
		{
			_Common C = new _Common();
			C.Abstract_Error() ;
		}

		public virtual long Get_Max_Size()
		// Gets current maximum cache size in bytes.
		{
			_Common C = new _Common();
			C.Abstract_Error() ;
			return 0 ;
		}

		public virtual long Get_Size()
		// Returns amount of currently cached data, in bytes.
		{
			_Common C = new _Common();
			C.Abstract_Error() ;
			return 0 ;
		}

		public virtual long Get_Stats( long Index,
		   ref int Address )
		{
			_Common C = new _Common();
			C.Abstract_Error() ;
			return 0 ;
		}

		public virtual bool Lock( long Address )
		/* Locks the data starting at the specified address into
		 memory so that it is not thrown away.  Returns false if
		 the specified data is not in the cache. */
		{
			_Common C = new _Common();
			C.Abstract_Error() ;
			return false ;
		}

		public virtual bool Unlock( long Address )
		/* Unlocks the data starting at the specified address in
		 memory.  Returns false if the specified data is not in
		 the cache, or if it is not locked. */
		{
			_Common C = new _Common();
			C.Abstract_Error() ;
			return false ;
		}

		public virtual long Get_Hits() // Total read hits
		{
			return Statistics.Hits ;
		}

		public virtual long Get_Misses() // Read misses
		{
			return Statistics.Misses ;
		}

		public virtual long Get_Writes() // Total writes
		{
			return Statistics.Writes ;
		}

		public virtual long Get_Turns()
		// Number of times blocks were thrown away to make room for more
		{
			return Statistics.Turns ;
		}

		public virtual long Get_Replacements() // Number of write hits
		{
			return Statistics.Replacements ;
		}

		public virtual void Set_Hits( long Value ) // Total read hits
		{
			Statistics.Replacements = Value ;
		}

		public virtual void Set_Misses( long Value ) // Read misses
		{
			Statistics.Misses = Value ;
		}

		public virtual void Set_Writes( long Value ) // Total writes
		{
			Statistics.Writes = Value ;
		}

		public virtual void Set_Turns( long Value )
		// Number of times blocks were thrown away to make room for more
		{
			Statistics.Turns = Value ;
		}

		public virtual void Set_Replacements( long Value ) // Number of write hits
		{
			Statistics.Replacements = Value ;
		}

		public long Hits
		{
		  get { return Get_Hits(); }
		  set { Set_Hits( value ); }
		}
		public long Misses
		{
		  get { return Get_Misses(); }
		  set { Set_Misses( value ); }
		}
		public long Replacements
		{
		  get { return Get_Replacements(); }
		  set { Set_Replacements( value ); }
		}
		public long Turns
		{
		  get { return Get_Turns(); }
		  set { Set_Turns( value ); }
		}
		public long Writes
		{
		  get { return Get_Writes(); }
		  set { Set_Writes( value ); }
		}
	} ; // TCache

	public class PCache : TCache
	{
	} ;


	public class TSCache64 : PSCache
	{
	} ;

	public class PSCache64 : TSCache64
	{
	} ;


	public class TSCache : TCache
	{
	} ;

	public class PSCache : TSCache
	{
	} ;


	public class TCOM_Cache : PSCache
	{
	} ;
	public class TCOM_Cache64 : TCOM_Cache
	{
	} ;
}





