/*
        Program Name : _Common
        Package Name : Subroutine Library - Layer 0
        Purpose      : Common base objects (TObject and TDelphi_Object)
        Institution  : Conroy & Conroy Co.
        Date Written : 19-Mar-2002
        Written By   : Alan Conroy
        Version      : 1.0

        Copyright (C) 2005-2021 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

           DATE        BY          REASON                         

        1-Nov-2002     EAC         Removed pointless overrides of
                                   TObject methods in BP.  Added
                                   reference counting support.

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************
*/

#ifndef _COMMON_HPP
	#define _COMMON_HPP

#include "eDefines.hpp"
#include <string>

using namespace std;

enum TMemory_Type{ MT_Unknown, /* Don't know */
                   MT_Heap, /* Allocated on heap */
                   MT_Data, /* "Global" data */
                   MT_Stack, /* "Local" data */
                   MT_Code /* executable */
                 } ;

class TDelphi_Object
{
						  public: // API...
							   virtual void
  #ifdef STDCALL_SUPPORTED
								   stdcall
  #endif
									   Attach()=0;

							   virtual bool
  #ifdef STDCALL_SUPPORTED
								   stdcall
  #endif
									   Detach()=0 ;

							   virtual bool
  #ifdef STDCALL_SUPPORTED
                                   stdcall
  #endif
                                       Is_Class( const string Name )=0 ;
} ; // TDelphi_Object

class TObject
{
                    public: // Constructor...
					   TObject();

  #ifdef CC_Debug
                    private: // Instance data...
                        string Sanity ;
  #endif

                    public: // API...
                        virtual void Attach() ;
                        virtual bool Detach() ;
                        virtual string Classname() ;
                        virtual bool Is_Class( const string Name ) ;
                        virtual TMemory_Type Get_Memory_Type() ;
} ; // TObject

#define PObject TObject*
#define PDelphi_Object TDelphi_Object*

void Abstract_Error() ;
#endif
