/*
		Program Name : _DebugIn
		Package Name : Subroutine Library - Layer 0
		Purpose      : Debugger Interface definitions
		Institution  : Conroy & Conroy Co.
		Date Written : 11-Jan-2001
		Written By   : Alan Conroy
		Version      : 1.0

		Copyright (C) 2001-2021 by Alan Conroy.  Released to the public domain.

		TO THE GLORY OF GOD THROUGH JESUS CHRIST

		*************************************************************
		*                                                           *
		*        M O D I F I C A T I O N   H I S T O R Y            *
		*                                                           *
		*************************************************************

			DATE        BY          REASON

		*************************************************************
		*                                                           *
		*        P R O G R A M   P U R P O S E                      *
		*                                                           *
		*************************************************************

		  TDebug_Interface is used when debugging code by providing a way of
		examining the internals of the software component at run-time (for
		production versions of software, this is often not supported).  The
		interface assumes a hierarchical form with the root of a component
		returning children representing internal objects.  Each of those objects
		may have children, and so forth.
*/

using System;
using _Common_unit;

namespace _DebugIn_unit
{
	public class TDebug_Interface : TObject
	{
		// API...
		public int Count()
		/* Returns the number of child components under
		  this level.  If this is a terminal (leaf)
		  node, the method returns 0. */
		{
			return 0;
		}


		public string Get_Title()
		/* Returns text representing this level (such as
		  "Cached data"). */
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return "";
		}


		public bool Has_Children()
		/* Returns whether or not there are child
		  components under this level.  If this is a
		  terminal (leaf) node, the method returns
		  false. */
		{
			return ( Count() != 0 ) ;
		}


		public void Set_Title( string Value )
		/* Sets text representing this level. */
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public void Activate()
		/* Indicates that the user wished to activate
		  this level.  The behavior is undefined and up
		  to the implementor.  Typically, this does
		  nothing unless this level is a terminal
		  representing some complex data which is not
		  viewable in a hierarchical form. */
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public bool Can_Change() // Returns True if this item can be modified.
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false ;
		}


		public TSDebug_Interface Child( int Ordinal )
		/* Returns a debug interface for the specified
		  child level (first child level is 0). */
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return null;
		}


		public void Kill() // Tells object to destroy itself */
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public string Modify( string Data, int Size )
		/* Modifies data.  If Size is -1, Data is assumed
		  to be null-terminated and its length is
		  automatically determined.  The result will be
		  nil or a pointer to a null string to indicate
		  success and error text otherwise. */
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return "";
		}


		public int Get_Context()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return 0;
		}


		public void Set_Context( int Value )
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
		}


		public bool Recursive()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false;
		}


		public bool Arbitrary_Length()
		{
			_Common C = new _Common() ;
			C.Abstract_Error() ;
			return false;
		}


		public int Context
		{
			get { return Get_Context(); }
			set { Set_Context( value ); }
		}

		public string Title
		{
			get { return Get_Title(); }
			set { Set_Title( value ); }
		}
	} ; // TSDebug_Interface


	public class TSDebug_Interface : TDebug_Interface
	{
	} ;

	
	public class PSDebug_Interface : TSDebug_Interface
	{
	} ;
}
