/*
        Program Name : _DebugIn
        Package Name : Subroutine Library - Layer 0
        Purpose      : Debugger Interface definitions
        Institution  : Conroy & Conroy Co.
        Date Written : 11-Jan-2001
        Written By   : Alan Conroy
        Version      : 1.0

	    Copyright (C) 2001-2021 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY          REASON

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          TDebug_Interface is used when debugging code by providing a way of
        examining the internals of the software component at run-time (for
        production versions of software, this is often not supported).  The
        interface assumes a hierarchical form with the root of a component
        returning children representing internal objects.  Each of those objects
        may have children, and so forth.
*/

#ifndef _DEBUGIN_HPP
	#define _DEBUGIN_HPP

#include "eDefines.hpp"
#include "_Common.hpp"
#include "_Types.hpp"

class TSDebug_Interface: public TObject
{
    public: /* API... */
                                virtual int32 Count() ;
                                /* Returns the number of child components under
                                  this level.  If this is a terminal (leaf)
                                  node, the method returns 0. */

                                virtual char* Get_Title() ;
      	                        /* Returns text representing this level (such as
                                  "Cached data"). */

                                virtual bool Has_Children() ;
                                /* Returns whether or not there are child
                                  components under this level.  If this is a
                                  terminal (leaf) node, the method returns
                                  false. */

                                virtual void Set_Title( char* Value ) ;
      	                        /* Sets text representing this level. */

                                virtual void Activate() ;
      	                        /* Indicates that the user wished to activate
                                  this level.  The behavior is undefined and up
                                  to the implementor.  Typically, this does
                                  nothing unless this level is a terminal
                                  representing some complex data which is not
                                  viewable in a hierarchical form. */

                                virtual bool Can_Change() ;
                                /* Returns True if this item can be modified. */

                                virtual TSDebug_Interface *Child( int32 Ordinal ) ;

      	                        /* Returns a debug interface for the specified
                                  child level (first child level is 0). */

                                virtual void Kill() ;
                                /* Tells object to destroy itself */

                        	    virtual char* Modify( char* Data, int Size ) ;
                                /* Modifies data.  If Size is -1, Data is assumed
                                  to be null-terminated and its length is
                                  automatically determined.  The result will be
                                  nil or a pointer to a null string to indicate
                                  success and error text otherwise. */

                                virtual int32 Get_Context() ;

                                virtual void Set_Context( int32 Value ) ;

                                virtual bool Recursive() ;

                                virtual bool Arbitrary_Length() ;
} ; // TSDebug_Interface

class TCOM_Debug_Interface
{
    public: // API...
									virtual int32
#ifdef _STDCALL_SUPPORTED
										stdcall
#endif
										Count()=0;
									/* Returns the number of child components under
                                      this level.  If this is a terminal (leaf)
                                      node, the method returns 0. */

									virtual char*
#ifdef _STDCALL_SUPPORTED
										stdcall
#endif
										Get_Title()=0 ;
                                    /* Returns text representing this level (such as
                                      "Cached data"). */

									virtual bool
#ifdef _STDCALL_SUPPORTED
										stdcall
#endif
										Has_Children()=0 ;
									/* Returns whether or not there are child components under
                                      this level.  If this is a terminal (leaf)
                                      node, the method returns false. */

									virtual void
#ifdef _STDCALL_SUPPORTED
										stdcall
#endif
										Set_Title( char* )=0 ;
									/* Sets text representing this level. */

									virtual void
#ifdef _STDCALL_SUPPORTED
										stdcall
#endif
										Activate()=0 ;
                                    /* Indicates that the user wished to activate
                                      this level.  The behavior is undefined and up
                                      to the implementor.  Typically, this does
                                      nothing unless this level is a terminal
                                      representing some complex data which is not
                                      viewable in a hierarchical form. */

									virtual bool
#ifdef _STDCALL_SUPPORTED
										stdcall
#endif
										Can_Change()=0 ;
									/* Returns True if this item can be modified. */

									virtual TCOM_Debug_Interface*
#ifdef _STDCALL_SUPPORTED
										stdcall
#endif
										Child( int32 )=0 ;
									/* Returns a debug interface for the specified
                                      child level (first child level is 0). */

									virtual void
#ifdef _STDCALL_SUPPORTED
										stdcall
#endif
										Kill()=0 ;
                                    /* Tells object to destroy itself */

									virtual char*
#ifdef _STDCALL_SUPPORTED
										stdcall
#endif
										Modify( char*, int )=0 ;
									/* Modifies data.  If Size is -1, Data is assumed
                                      to be null-terminated and its length is
                                      automatically determined.  The result will be
                                      nil or a pointer to a null string to indicate
                                      success and error text otherwise. */

									virtual int32
#ifdef _STDCALL_SUPPORTED
										stdcall
#endif
										Get_Context()=0 ;

									virtual void
#ifdef _STDCALL_SUPPORTED
										stdcall
#endif
										Set_Context( int32 Value )=0 ;

									virtual bool
#ifdef _STDCALL_SUPPORTED
										stdcall
#endif
										Recursive()=0 ;

									virtual bool
#ifdef _STDCALL_SUPPORTED
										stdcall
#endif
                                        Arbitrary_Length()=0 ;
} ; // TCOM_Debug_Interface

class TDebug_Interface: public TCOM_Debug_Interface{} ;

#define PDebug_Interface TDebug_Interface*

#define PSDebug_Interface TSDebug_Interface*

#endif
