/*
        Program Name : _Lib
        Package Name : Subroutine Library - Layer 1
        Purpose      : Abstract Object Librarian class definitions
        Institution  : Conroy & Conroy Co.
        Date Written : 15-Dec-1988
        Written By   : Alan Conroy
        Version      : 1.0

		    Copyright (C) 1988-2021 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *********************************************************
        *                                                       *
        *        M O D I F I C A T I O N   H I S T O R Y        *
        *                                                       *
        *********************************************************

             DATE      BY             REASON                   

		*********************************************************
        *                                                       *
        *           P R O G R A M   P U R P O S E               *
        *                                                       *
        *********************************************************

        *********************************************************
        *                                                       *
        *            F I L E   D E S C R I P T I O N S          *
        *                                                       *
		*********************************************************

*/

#ifndef _LIB_HPP
	#define _LIB_HPP

#include "eDefines.hpp"

#include "_Common.hpp" // TObject
#include "CommBase.hpp" // TBase_Delphi_Object
#include "COMInter.hpp" // TCommon_COM_Interface
#include "_Types.hpp" // int32
#include "_UE.hpp" // TUnified_Exception

#include <string>

using namespace std;

const LibErr_Facility = 14 ;
const LibErr_Success = 0 ;
const LibErr_Invalid_Library_File_Format = 1 ;
const LibErr_Module_Not_Found = 2 ;
const LibErr_Module_Already_Exists = 3 ;
const LibErr_Illegal_Module_Format = 4 ;

#define Block char[ 255 ] ;

struct TModule_Info
{
	char Name[ 128 ] ;
	int32 Size ;
	int32 Day ;
	int32 Month ;
	int32 Year ;
	int32 Hour ;
	int32 Minute ;
	int32 Flags ;
} ;

class TSObject_Librarian : public TObject
{
	public:
		int Creation_Hour ;
		int Creation_Minute ;
		int Creation_Day ;
		int Creation_Month ;
		int Creation_Year ;
		int Deleted_Space ;
		int Modified_Hour ;
		int Modified_Minute ;
		int Modified_Day ;
		int Modified_Month ;
		int Modified_Year ;
		string Version ;

	public: // API...
		virtual void Create_New( string Command, int32 Language_Id, int32 Typ ) ;
		virtual PSUnified_Exception Get_Last_Error() ;
		virtual void Extract( string Module, string Filename ) ;
		virtual void Extract_Index( int32 Module, string Filename, int32 Start_Block ) ;
		virtual void Open( string X ) ;
		virtual void Delete( string Command ) ;
		virtual void Insert( string Command, string Filename ) ;
		virtual void Get_Module( int32 Start_Block, int32 X, void* Buffer ) ;
		virtual bool Get_Module_Info( int32 X, TModule_Info &Info ) ;
		virtual int32 Size() ;
} ; // TSObject_Librarian

#define PSObject_Librarian TSObject_Librarian*

#ifdef STDCALL_SUPPORTED
class TCOM_Object_Librarian : public TCommon_COM_Interface
{
	public:
		virtual stdcall int32 Creation_Hour() = 0 ;
		virtual stdcall int32 Creation_Minute() = 0 ;
		virtual stdcall int32 Creation_Day() = 0 ;
		virtual stdcall int32 Creation_Month() = 0 ;
		virtual stdcall int32 Creation_Year() = 0 ;
		virtual stdcall int32 Deleted_Space() = 0 ;
		virtual stdcall int32 Modified_Hour() = 0 ;
		virtual stdcall int32 Modified_Minute() = 0 ;
		virtual stdcall int32 Modified_Day() = 0 ;
		virtual stdcall int32 Modified_Month() = 0 ;
		virtual stdcall int32 Modified_Year() = 0 ;
		virtual char* stdcall Version() = 0 ;
		virtual stdcall void Create_New( char* Command,
			int32 Language_Id, int32 Typ ) = 0 ;
		virtual PUnified_Exception stdcall Get_Last_Error() = 0 ;
		virtual stdcall void Extract( char* Module, char* Filename ) = 0 ;
		virtual PUnified_Exception stdcall Open( char* X ) = 0 ;
		virtual PUnified_Exception stdcall Delete( char* Command ) = 0 ;
		virtual PUnified_Exception stdcall Insert( char* Command,
			char* Filename ) = 0 ;
		virtual stdcall bool Get_Module_Info_By_Name( char* Command,
			TModule_Info& Info ) = 0 ;
		virtual stdcall bool Get_Module_Info_By_Index( int32 Index,
			TModule_Info& Info ) = 0 ;
		virtual stdcall int32 Size() = 0 ;
} ; // TCOM_Object_Librarian
#endif

class TObject_Librarian : public TBase_Delphi_Object
{
	public:
		virtual int32 Creation_Hour() = 0 ;
		virtual int32 Creation_Minute() = 0 ;
		virtual int32 Creation_Day() = 0 ;
		virtual int32 Creation_Month() = 0 ;
		virtual int32 Creation_Year() = 0 ;
		virtual int32 Deleted_Space() = 0 ;
		virtual int32 Modified_Hour() = 0 ;
		virtual int32 Modified_Minute() = 0 ;
		virtual int32 Modified_Day() = 0 ;
		virtual int32 Modified_Month() = 0 ;
		virtual int32 Modified_Year() = 0 ;
		virtual char* Version() = 0 ;
		virtual void Create_New( char* Command, int32 Language_Id,
			int32 Typ ) = 0 ;
		virtual PUnified_Exception Get_Last_Error() = 0 ;
		virtual void Extract( char* Module, char* Filename ) = 0 ;
		virtual PUnified_Exception Open( char* X ) = 0 ;
		virtual PUnified_Exception Delete( char* Command ) = 0 ;
		virtual PUnified_Exception Insert( char* Command, char* Filename ) = 0 ;
		virtual bool Get_Module_Info_By_Name( char* Command,
			TModule_Info& Info ) = 0 ;
		virtual bool Get_Module_Info_By_Index( int32 Index,
			TModule_Info& Info ) = 0 ;
		virtual int32 Size() = 0 ;
} ; // TObject_Librarian

#endif

