/*
		Program Name : _Loggers
		Package Name : Subroutine Library - Layer 1
		Purpose      : Generic logging class
		Institution  : Conroy & Conroy Co.
		Date Written : 26-April-2001
		Written By   : Alan Conroy
		Version      : 1.0B

		Copyright (C) 2001-2021 by Alan Conroy.  Released to the public domain.

		TO THE GLORY OF GOD THROUGH JESUS CHRIST

		*************************************************************
		*                                                           *
		*        M O D I F I C A T I O N   H I S T O R Y            *
		*                                                           *
		*************************************************************

			DATE        BY          REASON

		  1-Nov-2002    EAC         Fix compilation errors in BP.
		 20-Feb-2004                Convert to TSLogger.

		*************************************************************
		*                                                           *
		*        P R O G R A M   P U R P O S E                      *
		*                                                           *
		*************************************************************

		  TLogger is a generic logging interface.  Note that an integer
		  Level is passed to most methods.  The only important thing is that
		  the caller and responder have agreed on its meaning.
*/

using System;
using CommBase_unit; // TBase_Object

namespace _Loggers_unit
{
	public class _Loggers
	{
		// Message Logger Level mnuemonics...
		public const int LL_Undefined = 0 ;
		public const int LL_Fatal = 1 ;
		public const int LL_Serious = 2 ;
		public const int LL_Warning = 3 ;
		public const int LL_Hint = 4 ;
		public const int LL_Notification = 5 ;
	} ;

	public class TSLogger : TBase_Object
	{
		/* This procedure is called to log informational messages
		  to the code supplying the logger.  Mess is the message
		  text, Level indicates the type of message, and Abort
		  indicates whether or not to abort the process which
		  created the message.  If Abort is True upon return the
		  process should be terminated gracefully as quickly as possible. */
		public virtual bool Log_Message( string Mess, int Level )
		{
			return false ;
		}

		/* Same as Log_Message, but the message will be followed
		  by a Unnest_Message call when the condition causing the
		  message goes out of scope. */
		public virtual bool Log_Nested_Message( string Mess, int Level )
		{
			return false ;
		}

		/* This indicates the end of the condition under which the
		  previous message with the specified level occurred.
		  If Abort is True upon return, the process should be
		  terminated gracefully as quickly as possible. */
		public virtual bool Unnest_Message( int Level )
		{
			return false ;
		}

		/* This function is called during a long process to allow
		  for the user to abort it or for checking Windows
		  messages, etc.  If the function returns True, the
		  process should be terminated gracefully as quickly as possible. */
		public virtual bool Check_Terminate()
		{
			return false ;
		}
	} ;

	public class PSLogger : TSLogger
	{
	} ;
}

