/*
		Program Name : _Loggers
		Package Name : Subroutine Library - Layer 1
		Purpose      : Generic logging class
		Institution  : Conroy & Conroy Co.
		Date Written : 26-April-2001
		Written By   : Alan Conroy
		Version      : 1.0B

		Copyright (C) 2001-2021 by Alan Conroy.  Released to the public domain.

		TO THE GLORY OF GOD THROUGH JESUS CHRIST

		*************************************************************
		*                                                           *
		*        M O D I F I C A T I O N   H I S T O R Y            *
		*                                                           *
		*************************************************************

			DATE        BY          REASON

		  1-Nov-2002    EAC         Fix compilation errors in BP.
		 20-Feb-2004                Convert to TSLogger.

		*************************************************************
		*                                                           *
		*        P R O G R A M   P U R P O S E                      *
		*                                                           *
		*************************************************************

	  TLogger is a generic logging interface.  Note that an integer
	Level is passed to most methods.  The only important thing is that
	the caller and responder have agreed on its meaning.
*/

#ifndef _LOGGERS_HPP
	#define _LOGGERS_HPP

#include "eDefines.hpp"

#include "COMInter.hpp" // TCommon_COM_Interface
#include "CommBase.hpp" // TBase_Object

#include <string>

using namespace std;

// Message Logger Level constants...
const LL_Undefined = 0 ;
const LL_Fatal = 1 ;
const LL_Serious = 2 ;
const LL_Warning = 3 ;
const LL_Hint = 4 ;
const LL_Notification = 5 ;

class TSLogger : public TBase_Object
{
	public:
		/* This procedure is called to log informational messages
		to the code supplying the logger.  Mess is the message
		text, Level indicates the type of message, and Abort
		indicates whether or not to abort the process which
		created the message.  If Abort is True upon return the
		process should be terminated gracefully as quickly as
		possible. */
		virtual bool Log_Message( string Mess, int32 Level ) ;

		/* Same as Log_Message, but the message will be followed
		by a Unest_Message call when the condition causing the
		message goes out of scope. */
		virtual bool Log_Nested_Message( string Mess, int32 Level ) ;

		/* This indicates the end of the condition under which the
		previous message with the specified level occurred.
		If Abort is True upon return, the process should be
		terminated gracefully as quickly as possible. */
		virtual bool Unnest_Message( int32 Level ) ;

		/* This function is called during a long process to allow
		for the user to abort it or for checking Windows
		messages, etc.  If the function returns True, the
		process should be terminated gracefully as quickly as possible. */
		virtual bool Check_Terminate() ;
} ;


#ifdef STDCALL_SUPPORTED
class TCOM_Logger : public TCommon_COM_Interface
{
	public:
		virtual stdcall bool Log_Message( char* Mess, int32 Level ) = 0 ;

		virtual stdcall bool Log_Nested_Message( char* Mess, int32 Level ) = 0 ;

		virtual stdcall bool Unnest_Message( int32 Level ) = 0 ;

		virtual stdcall bool Check_Terminate() = 0 ;
} ;
#endif

#define PSLogger TSLogger*

#endif

