{
        Program Name : _Loggers
        Package Name : Subroutine Library - Layer 1
        Purpose      : Generic logging class
        Institution  : Conroy & Conroy Co.
        Date Written : 26-April-2001
        Written By   : Alan Conroy
        Version      : 1.0B

  	    Copyright (C) 2001-2021 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY          REASON

          1-Nov-2002    EAC         Fix compilation errors in BP.
         20-Feb-2004                Convert to TSLogger.

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

	      TLogger is a generic logging interface.  Note that an integer
	      Level is passed to most methods.  The only important thing is that
	      the caller and responder have agreed on its meaning.
}

unit _Loggers ;

interface

{$I EDEFINES.INC}

uses { Layer 0... }
     COMInter, { TCommon_COM_Interface }
     CommBase, { TBase_Object }
     _Types ; { int32 }

{ Message Logger Level mnuemonics... }
const LL_Undefined = 0 ;
const LL_Fatal = 1 ;
const LL_Serious = 2 ;
const LL_Warning = 3 ;
const LL_Hint = 4 ;
const LL_Notification = 5 ;

type TSLogger = object( TBase_Object )
                   public { API... }
                      { This procedure is called to log informational messages
                        to the code supplying the logger.  Mess is the message
                        text, Level indicates the type of message, and Abort
                        indicates whether or not to abort the process which
                        created the message.  If Abort is True upon return the
                        process should be terminated gracefully as quickly as
                        possible. }
                      function Log_Message( Mess : string ; Level : int32 ) : boolean ;
                         virtual ;

                      { Same as Log_Message, but the message will be followed
                        by a Unest_Message call when the condition causing the
                        message goes out of scope. }
                      function Log_Nested_Message( Mess : string ;
                          Level : int32 ) : boolean ; virtual ;

                      { This indicates the end of the condition under which the
                        previous message with the specified level occurred.
                        If Abort is True upon return, the process should be
                        terminated gracefully as quickly as possible. }
                      function Unnest_Message( Level : int32 ) : boolean ;
                          virtual ;

                      { This function is called during a long process to allow
                        for the user to abort it or for checking Windows
                        messages, etc.  If the function returns True, the
                        process should be terminated gracefully as quickly as
                        possible. }
                      function Check_Terminate : boolean ; virtual ;
               end ;

{$IFDEF DYNAMIC_OBJECTS}
type TCOM_Logger = class( TCommon_COM_Interface )
                       public { API... }
                          function Log_Message( Mess : PChar ; Level : int32 ) : boolean ;
                              virtual ; stdcall ; abstract ;

                          function Log_Nested_Message( Mess : PChar ;
                              Level : int32 ) : boolean ;
                              virtual ; stdcall ; abstract ;

                          function Unnest_Message( Level : int32 ) : boolean ;
                              virtual ; stdcall ; abstract ;

                          function Check_Terminate : boolean ;
                              virtual ; stdcall ; abstract ;
                   end ;
{$ENDIF}

     PSLogger = ^TSLogger ;

implementation

{ TSLogger methods... }

{ API... }

function TSLogger.Log_Message( Mess : string ; Level : integer ) : boolean ;

begin
    Log_Message := False ;
end ;


function TSLogger.Log_Nested_Message( Mess : string ; Level : integer ) : boolean ;

begin
    Log_Nested_Message := False ;
end ;


function TSLogger.Unnest_Message( Level : integer ) : boolean ;

begin
    Unnest_Message := False ;
end ;


function TSLogger.Check_Terminate : boolean ;

begin
    Check_Terminate := False ;
end ;



end.
