/*
        Program Name : _Propert
        Package Name : Subroutine Library - Layer 0
        Purpose      : Dynamic properties definition.
        Institution  : Conroy & Conroy Co.
        Date Written : 14-February-2005
        Written By   : Alan Conroy
        Version      : 1.0

	    Copyright (C) 2005 by Alan Conroy.  Released to the Public Domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY          REASON                         

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          TDynamic_Properties is an abstract base class definition for objects
        which provide dynamic object properties.
*/

#ifndef _PROPERT_HPP
	#define _PROPERT_HPP

#include "eDefines.hpp"

#include "_Common.hpp" // TObject
#include "_ComInter.hpp" // TCommon_COM_Object
#include "_GenVal.hpp" // TGeneric_Value
#include <string>

using namespace std;

class TSDynamic_Properties: public TObject
{
    public: // API...
                                    /* Adds a new property, returning the index.  Returns
                                      -1 if a property already exists by that name. */
                                    virtual int32 Add( const string Nam,
                                        PSGeneric_Value Value ) ;

                                    // Deletes all properties.
                                    virtual void Clear() ;

                                    // Count of defined properties...
                                    virtual int32 Count() ;

                                    /* Returns the index of the property with the
                                      specified name.  Returns -1 if the named
                                      property doesn't exist. */
                                    virtual int32 Index_Of( const string Nam ) ;

                                    /* Retrieves string name of property with
                                      given index.  If index is invalid, returns
                                      a null string. */
                                    virtual string Get_Name( int32 Index ) ;

                                    /* Gets the value for the specified index.  Returns
                                      nil if the index is invalid. */
                                    virtual PSGeneric_Value Get_Value( int32 Index ) ;

                                    // Sets the value for the specified index.
                                    virtual void Set_Value( int32 Index,
										PSGeneric_Value Value ) ;
} ;

#define PSDynamic_Properties TSDynamic_Properties*

class TDynamic_Properties: public TDelphi_Object
{
    public: // API...
                                    virtual int32 Add( const string Nam,
                                        TGeneric_Value* Value ) = 0 ;

                                    virtual void Clear() = 0 ;

                                    virtual int32 Count() = 0 ;

                                    virtual int32 Index_Of( const string Nam ) = 0 ;

                                    virtual string Get_Name( int32 Index ) = 0 ;

                                    virtual TGeneric_Value* Get_Value( int32 Index ) = 0 ;

                                    virtual void Set_Value( int32 Index,
                                        TGeneric_Value* Value ) = 0 ;
} ;

  #ifdef STDCALL_SUPPORTED
class TCOM_Dynamic_Properties: public TCommon_COM_Interface
{
    public: // API...
                                        virtual int32 stdcall Add( char* Nam,
                                            TCOM_Generic_Value *Value ) = 0 ;

                                        virtual void stdcall Clear() = 0 ;

                                        virtual int32 stdcall Count() = 0 ;

                                        virtual int32 stdcall Index_Of( char* Nam ) = 0 ;

                                        virtual char* stdcall Get_Name( int32 Index ) = 0 ;

                                        virtual TCOM_Generic_Value* stdcall Get_Value( int32 Index ) = 0 ;

                                        virtual void stdcall Set_Value( int32 Index,
                                            TCOM_Generic_Value* Value ) = 0 ;
} ;
  #endif

#endif
