/*
		Program Name : _Streams
		Package Name : Subroutine Library - Layer 0
		Purpose      : Stream object definitions
		Institution  : Conroy & Conroy Co.
		Date Written : 2002
		Written By   : Alan Conroy
        Version      : 1.0

		Copyright (C) 2002-2005 by Alan Conroy.  All rights reserved.

		TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
		*                                                           *
		*        M O D I F I C A T I O N   H I S T O R Y            *
		*                                                           *
		*************************************************************

			DATE        BY          REASON

		*************************************************************
		*                                                           *
		*        P R O G R A M   P U R P O S E                      *
		*                                                           *
		*************************************************************
*/


using System;
using _Common_unit;
using _UE_unit;

namespace _Streams_unit
{
	public abstract class TStream : TObject
	{
		// Returns True if there is no more data in the stream to read.
		public abstract bool At_End() ;

		// Returns last error code, and resets the error code.
		public abstract TUnified_Exception Last_Error() ;

		/* Reads the specified number of bytes from the stream.
		Size is modified to be the actual bytes transferred. */
		public abstract int Read( ref string Buffer, int _Size ) ;

		/* Reads one line of input, up to the size of the buffer.
		The line is assumed to end at an ASCII 13 code.  The
		ASCII 13 code is not included in the returned data.
		Size is modified to be the actual bytes transferred. */
		public abstract void Read_Line( ref string Buffer, ref int _Size ) ;

		// Position to specified byte within streamed data.
		public abstract void Seek( int Position ) ;

		/* Returns size of stream data, in bytes.  -1 indicate that
		the size is unknown or larger than 2^31. */
		public abstract int Size() ;

		/* Writes the specified buffer, of the specified size in
		bytes, to the stream. */
		public abstract void Write( ref string Buffer, int _Size ) ;

		/* Writes the specified null-terminated text to the stream.
		An ASCII code 13 is appended to the text on output. */
		public abstract void Write_Line( string Buffer ) ;
	} ;

	public class TSStream : TStream
	{
		// Returns True if there is no more data in the stream to read.
		public override bool At_End()
		{
			_Common C = new _Common();
			C.Abstract_Error() ;
			return true ;
		}

		// Returns last error code, and resets the error code.
		public override TUnified_Exception Last_Error()
		{
			return null ;
		}

		/* Reads the specified number of bytes from the stream.
		Size is modified to be the actual bytes transferred. */
		public override int Read( ref string Buffer, int _Size )
		{
			_Common C = new _Common();
			C.Abstract_Error() ;
			return 0 ;
		}

		/* Reads one line of input, up to the size of the buffer.
		The line is assumed to end at an ASCII 13 code.  The
		ASCII 13 code is not included in the returned data.
		Size is modified to be the actual bytes transferred. */
		public override void Read_Line( ref string Buffer, ref int _Size )
		{
			_Common C = new _Common();
			C.Abstract_Error() ;
		}

		// Position to specified byte within streamed data.
		public override void Seek( int Position )
		{
			_Common C = new _Common();
			C.Abstract_Error() ;
		}

		/* Returns size of stream data, in bytes.  -1 indicate that
		the size is unknown or larger than 2^31. */
		public override int Size()
		{
			_Common C = new _Common();
			C.Abstract_Error() ;
			return 0 ;
		}


		/* Writes the specified buffer, of the specified size in
		bytes, to the stream. */
		public override void Write( ref string Buffer, int _Size )
		{
			_Common C = new _Common();
			C.Abstract_Error() ;
		}

		// Writes the specified string to the stream.
		public virtual void Write_String( string Buffer )
		{
			_Common C = new _Common();
			C.Abstract_Error() ;
		}

		/* Writes the specified null-terminated text to the stream.
		An ASCII code 13 is appended to the text on output. */
		public override void Write_Line( string Buffer )
		{
			_Common C = new _Common();
			C.Abstract_Error() ;
		}

		/* Writes the specified string to the stream.
		An ASCII code 13 is appended to the text on output. */
		public virtual void Write_Line_String( string Buffer )
		{
			_Common C = new _Common();
			C.Abstract_Error() ;
		}
	} ; // TSStream
	
	public class PSStream : TSStream
	{
	} ;
}
