/*
        Program Name : _Streams
        Package Name : Subroutine Library - Layer 0
        Purpose      : Stream object definitions
        Institution  : Conroy & Conroy Co.
        Date Written : 2002
        Written By   : Alan Conroy
        Version      : 1.0

        Copyright (C) 2002-2005 by Alan Conroy.  All rights reserved.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY          REASON                         

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

*/

#ifndef _STREAMS_HPP
	#define _STREAMS_HPP

#include "eDefines.hpp"

#include "ComInter.hpp"
#include "_Common.hpp"
#include "_UE.hpp"
#include <string>

using namespace std;

#ifdef STDCALL_SUPPORTED
class TCOM_Stream: public TCommon_COM_Interface
{
    public: // API...
                          /* Returns True if there is no more data in the stream to
                            read. */
                          virtual bool stdcall At_End() = 0 ;

                          /* Reads the specified number of bytes from the stream.
                            Returns the actual bytes transferred in _Size. */
						  virtual int32 stdcall Read( void* Buffer, int32 &_Size ) = 0 ;

                          /* Reads one line of input, up to the size of the buffer.
                            The line is assumed to end at an ASCII 13 code.  The
                            ASCII 13 code is not included in the returned data.
                            Size is modified to be the actual bytes transferred. */
                          virtual void stdcall Read_Line( void* Buffer, int32 &_Size ) = 0 ;

                          /* Position to specified byte within streamed data. */
                          virtual void stdcall Seek( int32 Position ) = 0 ;

                          /* Returns size of stream data, in bytes.  -1 indicate that
                            the size is unknown or larger than 2^31. */
                          virtual int32 stdcall Size() = 0 ; 

                          /* Writes the specified buffer, of the specified size in
                            bytes, to the stream. */
                          virtual void stdcall Write( void* Buffer, int32 _Size ) = 0 ;

                          /* Writes the specified null-terminated text to the stream.
                            An ASCII code 13 is appended to the text on output. */
                          virtual void stdcall Write_Line( char* Buffer ) = 0 ;
} ;

class TStream: public TDelphi_Object
{
    public: // API...
                      /* Returns True if there is no more data in the stream to
                        read. */
                      virtual bool At_End() = 0 ;

                      /* Returns last error code, and resets the error code. */
					  virtual TUnified_Exception* Last_Error() = 0 ;

                      /* Reads the specified number of bytes from the stream.
                        Size is modified to be the actual bytes transferred. */
                      virtual int32 Read( void* Buffer, int32 _Size ) = 0 ;

                      /* Reads one line of input, up to the size of the buffer.
                        The line is assumed to end at an ASCII 13 code.  The
                        ASCII 13 code is not included in the returned data.
                        Size is modified to be the actual bytes transferred. */
                      virtual void Read_Line( void* Buffer, int32 &_Size ) = 0 ;

                      /* Position to specified byte within streamed data. */
                      virtual void Seek( int32 Position ) = 0 ;

                      /* Returns size of stream data, in bytes.  -1 indicate that
                        the size is unknown or larger than 2^31. */
                      virtual int32 Size() = 0 ;

                      /* Writes the specified buffer, of the specified size in
                        bytes, to the stream. */
                      virtual int32 Write( void* Buffer, int32 _Size ) = 0 ;

                      /* Writes the specified null-terminated text to the stream.
                        An ASCII code 13 is appended to the text on output. */
                      virtual void Write_Line( char* Buffer ) = 0 ;
} ;
#endif

class TSStream: public TObject
{
    public: // API...
                      /* Returns True if there is no more data in the stream to
                        read. */
                      virtual bool At_End() ;

                      /* Returns last error code, and resets the error code. */
                      virtual TSUnified_Exception *Last_Error() ;

                      /* Reads the specified number of bytes from the stream.
                        Size is modified to be the actual bytes transferred. */
                      virtual int32 Read( void* Buffer, int32 &_Size ) ;

                      /* Reads one line of input, up to the size of the buffer.
                        The line is assumed to end at an ASCII 13 code.  The
                        ASCII 13 code is not included in the returned data.
                        Size is modified to be the actual bytes transferred. */
                      virtual void Read_Line( void* Buffer, int32 &_Size ) ;

                      /* Reads one line of input.  The line is assumed to end at
                        an ASCII 13 code.  The ASCII 13 code is not included in
                        the returned data. */
                      virtual string Read_String() ;

                      /* Position to specified byte within streamed data. */
                      virtual void Seek( int32 Position ) ;

                      /* Writes the specified buffer, of the specified size in
                        bytes, to the stream. */
					  virtual void Write( void* Buffer, int32 _Size ) ;

                      /* Writes the specified string to the stream. */
                      virtual void Write_String( const string Buffer ) ;

					  /* Writes the specified null-terminated text to the stream.
                        An ASCII code 13 is appended to the text on output. */
                      virtual void Write_Line( char* Buffer ) ;

                      /* Writes the specified string to the stream.
                        An ASCII code 13 is appended to the text on output. */
                      virtual void Write_Line_String( const string Buffer ) ;
} ; // TSStream
     
#define PSStream TSStream*

#endif
